// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDSOAP_VERSION_H
#define KDSOAP_VERSION_H

#define KDSOAP_VERSION_STRING "2.2.0"
#define KDSOAP_VERSION_MAJOR 2
#define KDSOAP_VERSION_MINOR 2
#define KDSOAP_VERSION_PATCH 0
#define KDSOAP_VERSION KDSOAP_VERSION_CHECK(KDSOAP_VERSION_MAJOR, KDSOAP_VERSION_MINOR, KDSOAP_VERSION_PATCH)

/*
   for example: KDSOAP_VERSION >= KDSOAP_VERSION_CHECK(1, 2, 2))
*/
#define KDSOAP_VERSION_CHECK(major, minor, patch) ((major<<16)|(minor<<8)|(patch))

#endif
