/****************************************************************************
**
** This file is part of the KD Soap project.
**
** SPDX-FileCopyrightText: 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
**
** SPDX-License-Identifier: MIT
**
****************************************************************************/

#ifndef KDSOAP_H
#define KDSOAP_H

#include "KDSoapAuthentication.h"
#include "KDSoapClientInterface.h"
#include "KDSoapMessage.h"
#include "KDSoapNamespaceManager.h"
#include "KDSoapPendingCall.h"
#include "KDSoapPendingCallWatcher.h"
#include "KDSoapValue.h"

#endif
